// package de.deinpackage;

import java.io.*;
import java.text.*;


/*
 Page-Direktiven
 ---------------

 <%@ page isThreadSafe="false" %>                      soll Multithreading verhindern

 <%@ page errorPage="fehler.jsp" %>                    Fehlerseite (fr eine gute Fehlerseite stelle ich noch ein Beispiel bereit)

 <%@ page import="com.firma.niederlassung.abteilung.project.*" %>    import (Namensraum)

 Zu page gibt es noch folgende Attribute:
 language, extends, buffer, autoFlush, info, contentType, isErrorPage, session

 <%@ taglib uri = "appTLD" prefix="app" %>             Customer-Defined-Tags (extra Thema)

 <%@ include file = "copyright.html" %>                File-Include

 <%@ page isErrorPage="true" %>


 JSP-Tags:
 ---------

 <jsp:include page="pfad" flush="true"/>           Datei-Include
 <jsp:include page="<%= %>" flush="true"/>         zur Laufzeit ermittelt mglich


 <jsp:forward page="pfad"/>                        Forward wie bei Servlet


 <jsp:plugin />                                    Browser-Plugins


 <jsp:useBean />                                   Nutzung Java-Bean


 <jsp:getProperty />                               Bean-Getter


 <jsp:setProperty />                               Bean-Setter

 bean
 ----

 <jsp:useBean />                                   Nutzung Java-Bean
 <jsp:getProperty />                               Bean-Getter
 <jsp:setProperty />                               Bean-Setter

 - <jsp:useBean id="beanId" class="package.class" scope="page,request,session,application"/>
 -                   |
 -       +-----------+
 -       |
 -<%= beanId.getXy() %> Nutzung im Script-Code
 -       |     |
 -       |     +-----------------------------+
 -       +------------------+                |
 -                          |                |
 - <jsp:getProperty name="beanId" property="xy"/>
 -                          |                |
 - <jsp:setProperty name="beanId" property="xy" value="newXy"/>

 Achtung: Die Properties koennen Elementartypen oder Klassen aus java.lang.* sein.

 scope:
 page         - temporr
 request      - zur Auswertung im nchsten JSP/Servlet
 session      - Session
 application  - Singleton
 */


/**
 * rauspuzzeln &lt;% ... %&gt; - Tokens aus jsp-File
 */
public class JspCheck {

  /**
   * Merker fr Exit-Status ( 0 - keine Fehler )
   */
  private static int iExitStatus = 0;

  /**
   * Pfad zu den JSP-Dateien, die eingelesen werden sollen.
   */
  private static String jspDir;

  /**
   * Pfad f&uuml; die zu erzeugenden tempor&auml;ren Java-Klassen
   */
  private static String javaDir;

  /**
   * Package f&uuml; die zu erzeugenden tempor&auml;ren Java-Klassen
   */
  private static String jspcheckPackage;

  /**
   * Main-Methode dieser Klasse. <br/>
   * Beginnt im {@link #jspDir}-Verzeichnis mit der Suche nach <b>*.jsp</b>-Dateien.<br/>
   * @param args String-Array mit den Kommandozeilenparametern
   */
  public static void main(String[] args) {
    System.out.println("noch jsp:getProperty und jsp:setProperty einbauen");
    System.out.println("noch @import einbauen");
    if (args.length < 3) {
      System.out.println("JspCheck kopiert die <% ... %> Tokens aus einer JSP-Datei in eine *_jsp.java-Datei ");
      System.out.println("start with");
      System.out.println("JspCheck jspDirectory javaDirectory jspcheckPackage");
      // Rckgabe Fehlerstatus und Programmende
      // Bei Start aus ANT im Tag javac muss mit dem Parameter fork="true" eine neue JVM gestartet werden
      // sonst System.exit(int) auskommentieren, weil System.exit(int) ANT komplett abbricht
      System.exit(1); // Rckgabe: nicht ok
    }

    jspDir = correctFileSeparator(args[ 0 ]);
    javaDir = correctFileSeparator(args[ 1 ]);
    jspcheckPackage = args[ 2 ];

    System.out.println("JspCheck Start in " + jspDir + ", generate *_jsp.java files to " + javaDir + " in package "
        + jspcheckPackage);
    scanDir(jspDir);

    // Rckgabe Fehlerstatus und Programmende
    // Bei Start aus ANT im Tag javac muss mit dem Parameter fork="true" eine neue JVM gestartet werden
    // sonst System.exit(int) auskommentieren, weil System.exit(int) ANT komplett abbricht
    System.exit(iExitStatus);
  }// end method main

  /**
   * Methode zum Durchlaufen des aktuellen Verzeichnisses und
   * aller Unterverzeichnisse. <br/>
   * Ruft fr alle Dateien die Methode {@link #processFile} auf.
   *
   * @param dirPath Pfad des zu durchsuchenden Verzeichnisses
   */
  private static void scanDir(String dirPath) {
    // System.out.println( "scanDir: " + dirPath ) ;

    // Ermittle Liste Dateien
    File[] fileArr = new File(dirPath).listFiles();

    for (int i = 0; i < fileArr.length; i++) {
      if (fileArr[i].isDirectory()) {
        scanDir(fileArr[i].getParent() + File.separator + fileArr[i].getName());
      }
      else { // ein File
        processFile(fileArr[i].getParent() + File.separator + fileArr[i].getName());
      }
    } // end for
  }// end method scanDir

  /**
   * Methode zum Abarbeiten aller Dateien, ausser Verzeichnisse. <br/>
   * Diese Methode wird von {@link #scanDir} fr jede Datei<br/>
   * aufgerufen.<br/>
   * Sie ruft fr jede <b>*.jsp</b>-Datei die Methode {@link #processJspFile} auf.<br/>
   * Sie ruft fr jede <b>package.jspcheck</b>-Datei die Methode {@link # ??} auf.<br/>
   *
   * @param fileName Dateiname mit Pfad und Endung
   */
  private static void processFile(String fileName) {
    // System.out.println( "processFile: " + fileName ) ;
    if (fileName.endsWith(".jsp")) {
      processJspFile(fileName);
    }// end if
  }// end method processFile

  /**
   * Methode zum Abarbeiten einer <b>*.jsp</b>-Datei. <br/>
   * Diese Methode wird von {@link #processFile} fr jede
   * <b>*.jsp</b>-Datei aufgerufen.<br/>
   * Sie benutzt die Methode {@link # ???} . <br/>
   *
   * @param strPaFileName Dateiname mit Pfad und Endung
   */
  public static void processJspFile(String strPaFileName) {
    String strJsp = file2str(strPaFileName);
    StringBuffer strBuffCjspHeader = new StringBuffer();
    StringBuffer strBuffCjspBody = new StringBuffer();

    for (int i = 0; i < strJsp.length() - 1; i++) {
      if (strJsp.charAt(i) == '<') {
        // Achtung, charAt funzt nur wenn vor dem Stringende aufgehrt wird, sonst StringOutOfBoundsException
        if (strJsp.charAt(i) == '<' && strJsp.charAt(i + 1) == '%') {
          i += 2;
          StringBuffer strBuffTok = new StringBuffer();

          // Achtung, charAt funzt nur wenn vor dem Stringende aufgehrt wird, sonst StringOutOfBoundsException
          // for (; i < strJsp.length() && !(strJsp.indexOf("%>", i) == i); i++) {
          for (; i < strJsp.length() - 1 && !(strJsp.charAt(i) == '%' && strJsp.charAt(i + 1) == '>'); i++) {
            strBuffTok.append(strJsp.charAt(i));
          }// end for
          System.out.println("Gelesen: " + strBuffTok);
          String strTok = strBuffTok.toString().trim();

          if (strTok.indexOf("//") > -1) {
            // ein Inline-Kommentar ist im Token aufgetaucht (hoffentlich nicht // in einem String)
            // Zeilen-Umbruch, damit der nchste Code nicht im Kommentar landet
            strTok += "\n";
          }

          if (strTok.startsWith("--")) {
            // Scriptlet-Kommentar <%-- ... --%>
            // wird nicht mit bernommen, weil eventuell java-Kommentare drin sind
            // ein *_/ wrde andere Kommentare beenden
            System.out.println("Script-Kommentar <%-- ... --%> wird nicht gespeichert");
            strTok = "\n";
          }
          else if (strTok.startsWith("=")) {
            System.out.println("Script-out <%= ... %> wird auf System.out umgeleitet");
            strTok = "System.out.print(" + strTok.substring(1) + ");" + "\n";
          }// end if

          // import declaration
          else if (startsWithAtPage(strTok)) {
            // string startet mit @ page
            if (startsWithAtPageImport(strTok)) {
              // string startet mit @ page import
              strTok = cutLeftAtPageImportEqual(strTok);
              strTok = "import " + strTok.replace('"', ' ') + ";";
              System.out.println("Ausgeben in Header: " + strTok);
              strBuffCjspHeader.append(strTok + "\n");
              strTok = ""; // nicht in method body uebernehmen
            }
            else {
              strTok = ""; // andere page directiven nicht bernehmen
            }
            // i += 2;
          }// end else if
          else if (strTok.startsWith("@")) {
            // alle anderen @-Tags ignorieren
            strTok = ""; // nicht in method body uebernehmen
          }// end else if
          System.out.println("Ausgeben: " + strTok);
          strBuffCjspBody.append(strTok + "\n");
        }// end if

        // <jsp:useBean id="beanId" class="package.class" scope="page,request,session,application"/>
        else if (strJsp.charAt(i) == '<' && strJsp.charAt(i + 1) == 'j' && // strJsp.charAt(i+2) == 's' &&
            // strJsp.charAt(i+3) == 'p' &&
            // strJsp.charAt(i+4) == ':' &&
            // strJsp.charAt(i+5) == 'u' &&
            strJsp.indexOf("<jsp:useBean ", i) == i) {
          // i += 2;
          StringBuffer strBuffTok = new StringBuffer();

          for (; i < strJsp.length() && !(strJsp.indexOf("/>", i) == i); i++) {
            strBuffTok.append(strJsp.charAt(i));
          }// end for
          // System.out.println(strBuffTok);
          String strTok = strBuffTok.toString().trim();

          strTok = cutLeft(strTok, "<jsp:useBean ").trim();
          // System.out.println(strTok);
          String strId = "";

          if (strTok.startsWith("id=")) {
            strTok = cutLeft(strTok, "id=\"");
            // System.out.println(strTok);
            strId = strTok.substring(0, strTok.indexOf("\""));
            // System.out.println(strId);
            strTok = cutLeft(strTok, strId + "\"").trim();
            // System.out.println(strTok);
          }
          String strClass = "";

          if (strTok.startsWith("class=")) {
            strTok = cutLeft(strTok, "class=\"");
            // System.out.println(strTok);
            strClass = strTok.substring(0, strTok.indexOf("\""));
            // System.out.println(strClass);
            strTok = cutLeft(strTok, strClass + "\"").trim();
            // System.out.println(strTok);
          }
          // scope wird vorerst ignoriert

          strTok = strClass + " " + strId + "=(" + strClass + ")session.getAttribute(\"" + strId + "\");" + "\n";
          System.out.println("Ausgeben: " + strTok);
          strBuffCjspBody.append(strTok + "\n");
          i += 2;
        }// end else if

        // <jsp:getProperty name="beanId" property="xy"/>
        else if (strJsp.charAt(i) == '<' && strJsp.charAt(i + 1) == 'j' && // strJsp.charAt(i+2) == 's' &&
            // strJsp.charAt(i+3) == 'p' &&
            // strJsp.charAt(i+4) == ':' &&
            // strJsp.charAt(i+5) == 'g' &&
            strJsp.indexOf("<jsp:getProperty ", i) == i) {
          // i += 2;
          StringBuffer strBuffTok = new StringBuffer();

          for (; i < strJsp.length() && !(strJsp.indexOf("/>", i) == i); i++) {
            strBuffTok.append(strJsp.charAt(i));
          }// end for
          // System.out.println(strBuffTok);
          String strTok = strBuffTok.toString().trim();

          strTok = cutLeft(strTok, "<jsp:getProperty ").trim();
          // System.out.println(strTok);
          String strName = "";

          if (strTok.startsWith("name=")) {
            strTok = cutLeft(strTok, "name=\"");
            // System.out.println(strTok);
            strName = strTok.substring(0, strTok.indexOf("\""));
System.out.println("strName " + strName);
            strTok = cutLeft(strTok, strName + "\"").trim();
            // System.out.println(strTok);
          }
          String strProperty = "";

          if (strTok.startsWith("property=")) {
            strTok = cutLeft(strTok, "property=\"");
            // System.out.println(strTok);
            strProperty = strTok.substring(0, strTok.indexOf("\""));
            // System.out.println(strClass);
            strTok = cutLeft(strTok, strProperty + "\"").trim();
System.out.println("strProperty " + strProperty);
          }

strTok = strName+".get"+capitalize(strProperty)+"();" ; // beanId.getXy()
          System.out.println("Ausgeben: " + strTok);
          strBuffCjspBody.append(strTok + "\n");
          i += 2;
        }// end else if

        // <jsp:setProperty    name="beanId" property="xy" value="newXy"/>
        else if (strJsp.charAt(i) == '<' && strJsp.charAt(i + 1) == 'j' && // strJsp.charAt(i+2) == 's' &&
            // strJsp.charAt(i+3) == 'p' &&
            // strJsp.charAt(i+4) == ':' &&
            // strJsp.charAt(i+5) == 's' &&
            strJsp.indexOf("<jsp:setProperty ", i) == i) {
          // i += 2;
          StringBuffer strBuffTok = new StringBuffer();

          for (; i < strJsp.length() && !(strJsp.indexOf("/>", i) == i); i++) {
            strBuffTok.append(strJsp.charAt(i));
          }// end for
          // System.out.println(strBuffTok);
          String strTok = strBuffTok.toString().trim();

          strTok = cutLeft(strTok, "<jsp:setProperty ").trim();
          // System.out.println(strTok);
          String strName = "";

          if (strTok.startsWith("name=")) {
            strTok = cutLeft(strTok, "name=\"");
            // System.out.println(strTok);
            strName = strTok.substring(0, strTok.indexOf("\""));
System.out.println("strName " + strName);
            strTok = cutLeft(strTok, strName + "\"").trim();
            // System.out.println(strTok);
          }
          String strProperty = "";
          if (strTok.startsWith("property=")) {
            strTok = cutLeft(strTok, "property=\"");
            // System.out.println(strTok);
            strProperty = strTok.substring(0, strTok.indexOf("\""));
            // System.out.println(strClass);
            strTok = cutLeft(strTok, strProperty + "\"").trim();
System.out.println("strProperty " + strProperty);
          }
          String strValue = "";
          if (strTok.startsWith("value=")) {
            strTok = cutLeft(strTok, "value=\"");
            // System.out.println(strTok);
            strValue = strTok.substring(0, strTok.indexOf("\""));
            // System.out.println(strClass);
            strTok = cutLeft(strTok, strValue + "\"").trim();
System.out.println("strProperty " + strValue);
          }

// falls der Parameter-Typ nicht String, aber Elementartyp oder Klasse aus dem Package java.lang ist,
// wird der Parameter mit (paramClass).valueOf( String-Parameter ) konvertiert
// in diser Version nicht ntersttzt

strTok = strName+".set"+capitalize(strProperty)+"(\""+strValue+"\");" ; // beanId.setXy("newXy")
          System.out.println("Ausgeben: " + strTok);
          strBuffCjspBody.append(strTok + "\n");
          i += 2;
        }// end else if

      }
    }// end for
    writeJavaFile(strBuffCjspHeader.toString(), strBuffCjspBody.toString(),
        strPaFileName.substring(0, strPaFileName.lastIndexOf(File.separator) + 1)
        + encodeFileNameToClassName(strPaFileName.substring(strPaFileName.lastIndexOf(File.separator) + 1)), strPaFileName);
  }// end method processJspFile

  /**
   * Schreiben der tempor&auml;ren Java-Datei f&uuml;r den <b>jspcheck</b>
   *
   * @param strPaJava zu schreibender <b>javadoc</b>-Kommentar
   * @param strPaFileName Name der schreibenden Datei mit Pfad und ohne Endung
   * @param strPaOriginalFileName Original-Dateiname fr eventuelle Fehlermeldung
   */
  private static void writeJavaFile(String strPaJavaHeader, String strPaJavaBody, String strPaFileName, String strPaOriginalFileName) {
    strPaJavaHeader += "import javax.servlet.*;\n" + "import javax.servlet.http.*;\n" + "import javax.servlet.jsp.*;\n";

    strPaOriginalFileName = strPaOriginalFileName.replace('\\', '/');// encode Backslash zu Slash wegen Problemen mit Erkennung von Backslashes als ESC-Sequenzen in java
    strPaJavaHeader += "/**\n * (Class generated by <b>jspcheck</b> from <b>" + encodeHtml(strPaOriginalFileName)
        + "</b>).\n*/";

    String outClassWithPackageWithSlashs = jspcheckPackage.replace('.', File.separatorChar) + cutLeft(strPaFileName, jspDir);

    String javaFileStr = "package "
        + cutLeft(cutLeft((outClassWithPackageWithSlashs).replace(File.separatorChar, '.'), "."), ".");
    String className = javaFileStr.substring(javaFileStr.lastIndexOf(".") + 1);

    javaFileStr = javaFileStr.substring(0, javaFileStr.lastIndexOf(".")) + ";\n\n";
    javaFileStr += strPaJavaHeader + "\n";
    javaFileStr += "public class " + className + " {\n";
    javaFileStr += "    public void _jspService(HttpServletRequest request, HttpServletResponse  response)\n"
        + "        throws java.io.IOException, ServletException {\n\n" + "        JspFactory _jspxFactory = null;\n"
        + "        PageContext pageContext = null;\n" + "        HttpSession session = null;\n"
        + "        ServletContext application = null;\n" + "        ServletConfig config = null;\n"
        + "        JspWriter out = null;\n" + "        Object page = this;\n" + "        String  _value = null;\n"
        + "        try {\n";

    javaFileStr += strPaJavaBody + "\n        } catch (Throwable t) {}\n    }\n}";

    // System.out.println( "javaFileStr " + javaFileStr ) ;
    str2file(javaFileStr, javaDir + File.separatorChar + outClassWithPackageWithSlashs + ".java");
  }// end method writeJavaFile

  /**
   * Einlesen einer Datei in einen String
   *
   * @param  fileName Name der einzulesenden Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(String fileName) {
    return file2str(new File(fileName));
  } // end method file2str

  /**
   * Einlesen einer Datei in einen String
   *
   * @param inFile einzulesende Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(File inFile) {
    String  strReturn = null;

    try {
      FileReader inFileReader = new FileReader(inFile);
      char[] cArr = new char[ (int) inFile.length() ];

      inFileReader.read(cArr, 0, cArr.length);
      strReturn = new String(cArr);
    } catch (Exception e) {
      System.err.println(e.getMessage());
      e.printStackTrace();
    }
    return strReturn;
  }// end method file2str

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param fileName Name der Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file(String outStr, String fileName) {
    return str2file(outStr, new File(fileName));
  } // end method str2file

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param outFile  Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file(String outStr, File outFile) {
    boolean bReturn = false;

    try {
      if (outFile.exists()) {
        outFile.delete();
      }

      createNewFile(outFile);

      FileWriter  outFileWriter = new FileWriter(outFile);

      outFileWriter.write(outStr);
      outFileWriter.close();

      bReturn = true;
    } catch (Exception e) {
      System.err.println(e.getMessage());
      e.printStackTrace();
    }

    return bReturn;
  } // end method str2file

  /**
   * Erzeugen einer neuen Datei . <br/>
   * Eine eventuell vorhandene gleichnamige Datei wird vorher gelscht.<br/>
   * Falls notwendig, wird das Verzeichnis fr die Datei angelegt.<br/>
   *
   * @param filePa anzulegende Datei
   * @throws IOException weiterwerfen Exception vom Erzeugen der Datei
   */
  private static void createNewFile(File filePa)
      throws IOException {
    if (filePa.exists()) {
      filePa.delete();
    }

    try {
      filePa.createNewFile();
    } catch (IOException ioe) {
      // Versuch Verzeichnis anzulegen
      (new File(filePa.getParent())).mkdirs();
      // zweiter Versuch
      filePa.createNewFile();
    }// end catch
  }// end method

  /**
   * Teilstring von String links abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutLeft(String srcStr, String cutStr) {
    if (srcStr == null || cutStr == null) {
      return srcStr;
    }

    String  retStr = srcStr;

    if (retStr.length() >= cutStr.length() && retStr.startsWith(cutStr)) {
      retStr = retStr.substring(cutStr.length()); // cutStr abschneiden
    }

    return retStr;
  }// end method cutLeft

  /**
   * Teilstring von String rechts abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  private static String cutRight(String srcStr, String cutStr) {
    if (srcStr == null || cutStr == null) {
      return srcStr;
    }

    String  retStr = srcStr;

    if (retStr.length() >= cutStr.length() && retStr.endsWith(cutStr)) {
      retStr = retStr.substring(0, retStr.length() - cutStr.length());  // cutStr abschneiden
    }

    return retStr;
  }// end method cutRight

  /**
   * Korrigiert im per Parameter &uuml;bergebenen Pfad-String die
   * File-Separatoren damit die Ausschneid- und Ersetzungsoperationen
   * klappen . <br/>
   *
   * @param strParam zu korrigierender Pfad-String
   * @return korrigierter Pfad-String
   */
  private static String correctFileSeparator(String strParam) {
    if (File.separatorChar == '/') {
      return strParam.replace('\\', '/');
    }
    else if (File.separatorChar == '\\') {
      return strParam.replace('/', '\\');
    }
    return strParam;
  }// end method correctFileSeparator(

  /**
   * Filter the specified string for characters that are senstive to
   * HTML interpreters, returning the string with these characters replaced
   * by the corresponding character entities.
   *
   * @param value The string to be filtered and returned
   * @return the filtered String
   */
  private static final String encodeHtml(String value) {
    if (value == null) {
      return (null);
    }
    char  content[] = new char[value.length()];

    value.getChars(0, value.length(), content, 0);
    StringBuffer  result = new StringBuffer(content.length + 50);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {
      case '<':
        result.append("&lt;");
        break;

      case '>':
        result.append("&gt;");
        break;

      case '&':
        result.append("&amp;");
        break;

      case '"':
        result.append("&quot;");
        break;

      default:
        result.append(content[i]);
      }
    }
    return (result.toString());
  }// end method encodeHtml

  /**
   * Umwandeln eines &uuml;bergebenen Dateinamens in einen zul&auml;sigen Java-Klassen-Namen. <br/>
   * Alle verbotenen Zeichen (zum Beispiel Punkt) werden durch einen Unterstrich ersetzt.<br/>
   * Wenn der Name mit einer Ziffer beginnt, wird ein Unterstrich vor den Namen gesetzt.<br/>
   *
   * @param umzwandelnder Dateiname
   * @param erzeugter Klassen-Name
   */
  private static String encodeFileNameToClassName(String strParam) {
    if (strParam == null) {
      return (null);
    }
    char  content[] = new char[strParam.length()];

    strParam.getChars(0, strParam.length(), content, 0);
    StringBuffer  result = new StringBuffer(content.length + 50);

    for (int i = 0; i < content.length; i++) {
      if (i == 0 && Character.isDigit(content[i])) {
        // Ziffer am Anfang
        result.append("_");
        result.append(content[i]);
      }
      else if ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".indexOf(Character.toUpperCase(content[i])) < 0) {
        // kein erlaubtes Zeichen
        result.append("_");
      }
      else {
        result.append(content[i]);
      }
    }
    return (result.toString());
  }// end method encodeFileNameToClassName

  /**
   * Test String auf Beginn mit @page
   */
  private static boolean startsWithAtPage(String paStr) {
    String tmpStr = paStr.trim();

    // System.out.println("startsWithAtPage tmpStr1:"+tmpStr);
    if (tmpStr.startsWith("@")) {
      // System.out.println("startsWithAtPage tmpStr2:"+tmpStr);
      tmpStr = cutLeft(tmpStr, "@").trim();
      // System.out.println("startsWithAtPage tmpStr3:"+tmpStr);
      if (tmpStr.startsWith("page ")) {
        // System.out.println("startsWithAtPage true");
        return true;
      }
    }
    return false;
  }// end method startsWithAtPage

  /**
   * Test String auf Beginn mit @page import
   */
  private static boolean startsWithAtPageImport(String paStr) {
    String tmpStr = paStr.trim();

    // System.out.println("startsWithAtPageImport tmpStr1:"+tmpStr);
    if (tmpStr.startsWith("@")) {
      // System.out.println("startsWithAtPageImport tmpStr2:"+tmpStr);
      tmpStr = cutLeft(tmpStr, "@").trim();
      // System.out.println("startsWithAtPageImport tmpStr3:"+tmpStr);
      if (tmpStr.startsWith("page ")) {
        // System.out.println("startsWithAtPageImport tmpStr4:"+tmpStr);
        tmpStr = cutLeft(tmpStr, "page ").trim();
        // System.out.println("startsWithAtPageImport tmpStr5:"+tmpStr);
        if (tmpStr.startsWith("import")) {
          // System.out.println("startsWithAtPageImport true");
          return true;
        }
      }
    }
    return false;
  }// end method startsWithAtPageImport

  /**
   * String-Anfang "@page import =" abschneiden
   */
  private static String cutLeftAtPageImportEqual(String paStr) {
    return cutLeft(cutLeft(cutLeft(cutLeft(paStr.trim(), "@").trim(), "page ").trim(), "import").trim(), "=");
  }// end method cutLeftAtPageImportEqual


  /**
   * ersten Buchstaben im String gro machen, Capitalize
   *
   * @param strPaLower zu bearbeitender String
   * @return kapitalisierter String
   */
  public static final String capitalize(String strPaLower) {
    String  retString = null;

    if (strPaLower != null) {
      if (strPaLower.length() < 2) {
        retString = strPaLower.toUpperCase();
      }
      else {
        retString = strPaLower.substring(0, 1).toUpperCase();
        retString += strPaLower.substring(1);
      }
    }

    return retString;
  } // end method capitalize

}// end class JspCheck
